@@ -3,8 +3,8 @@
 -- *** ADAPTATION ALERT ***
 -- OGC will not examine this script for adaptations.
 -- Please add any other cleanup to this script.
--- 
-DELETE FROM spatial_ref_sys;
+--
+--DELETE FROM spatial_ref_sys;
 DELETE FROM geometry_columns;
 DROP TABLE lakes;
 DROP TABLE road_segments;
