@@ -30,7 +30,6 @@
 
 #include <stdio.h>
 #include <assert.h>
-#include "../postgis_revision.h"
 
 #define xstr(s) str(s)
 #define str(s) #s
@@ -223,7 +222,7 @@ getPoint3dz_p(const POINTARRAY *pa, uint32_t n, POINT3DZ *op)
 		return 0;
 	}
 
-	//assert(n < pa->npoints); --causes point emtpy/point empty to crash
+	//assert(n < pa->npoints); --causes point empty/point empty to crash
 	if ( n>=pa->npoints )
 	{
 		lwnotice("%s [%d] called with n=%d and npoints=%d", __FILE__, __LINE__, n, pa->npoints);
@@ -444,33 +443,30 @@ void printPA(POINTARRAY *pa)
 	POINT4D pt;
 	char *mflag;
 
-
+	if (!pa)
+	{
+		lwnotice("                    PTARRAY is null pointer!");
+		return;
+	}
 	if ( FLAGS_GET_M(pa->flags) ) mflag = "M";
 	else mflag = "";
 
 	lwnotice("      POINTARRAY%s{", mflag);
-	lwnotice("                 ndims=%i,   ptsize=%i",
+	lwnotice("                 ndims=%i,   ptsize=%zu",
 	         FLAGS_NDIMS(pa->flags), ptarray_point_size(pa));
-	lwnotice("                 npoints = %i", pa->npoints);
+	lwnotice("                 npoints = %u", pa->npoints);
 
-	if (!pa)
+	for (t = 0; t < pa->npoints; t++)
 	{
-		lwnotice("                    PTARRAY is null pointer!");
+		getPoint4d_p(pa, t, &pt);
+		if (FLAGS_NDIMS(pa->flags) == 2)
+			lwnotice("                    %i : %lf,%lf", t, pt.x, pt.y);
+		if (FLAGS_NDIMS(pa->flags) == 3)
+			lwnotice("                    %i : %lf,%lf,%lf", t, pt.x, pt.y, pt.z);
+		if (FLAGS_NDIMS(pa->flags) == 4)
+			lwnotice("                    %i : %lf,%lf,%lf,%lf", t, pt.x, pt.y, pt.z, pt.m);
 	}
-	else
-	{
 
-		for (t = 0; t < pa->npoints; t++)
-		{
-			getPoint4d_p(pa, t, &pt);
-			if (FLAGS_NDIMS(pa->flags) == 2)
-				lwnotice("                    %i : %lf,%lf", t, pt.x, pt.y);
-			if (FLAGS_NDIMS(pa->flags) == 3)
-				lwnotice("                    %i : %lf,%lf,%lf", t, pt.x, pt.y, pt.z);
-			if (FLAGS_NDIMS(pa->flags) == 4)
-				lwnotice("                    %i : %lf,%lf,%lf,%lf", t, pt.x, pt.y, pt.z, pt.m);
-		}
-	}
 	lwnotice("      }");
 }
 
@@ -649,9 +645,7 @@ deparse_hex(uint8_t str, char *result)
 void
 interpolate_point4d(const POINT4D *A, const POINT4D *B, POINT4D *I, double F)
 {
-#if PARANOIA_LEVEL > 0
-	if (F < 0 || F > 1) lwerror("interpolate_point4d: invalid F (%g)", F);
-#endif
+	assert(F >= 0 && F <= 1);
 	I->x=A->x+((B->x-A->x)*F);
 	I->y=A->y+((B->y-A->y)*F);
 	I->z=A->z+((B->z-A->z)*F);
