@@ -32,7 +32,7 @@ static uint8_t lwgeom_twkb_type(const LWGEOM *geom)
 {
 	uint8_t twkb_type = 0;
 
-	LWDEBUGF(2, "Entered  lwgeom_twkb_type",0);
+	LWDEBUG(2, "Entered lwgeom_twkb_type");
 
 	switch ( geom->type )
 	{
@@ -42,6 +42,7 @@ static uint8_t lwgeom_twkb_type(const LWGEOM *geom)
 		case LINETYPE:
 			twkb_type = WKB_LINESTRING_TYPE;
 			break;
+		case TRIANGLETYPE:
 		case POLYGONTYPE:
 			twkb_type = WKB_POLYGON_TYPE;
 			break;
@@ -54,12 +55,12 @@ static uint8_t lwgeom_twkb_type(const LWGEOM *geom)
 		case MULTIPOLYGONTYPE:
 			twkb_type = WKB_MULTIPOLYGON_TYPE;
 			break;
+		case TINTYPE:
 		case COLLECTIONTYPE:
 			twkb_type = WKB_GEOMETRYCOLLECTION_TYPE;
 			break;
 		default:
-			lwerror("Unsupported geometry type: %s [%d]",
-				lwtype_name(geom->type), geom->type);
+			lwerror("%s: Unsupported geometry type: %s", __func__, lwtype_name(geom->type));
 	}
 	return twkb_type;
 }
@@ -112,6 +113,7 @@ static int ptarray_to_twkb_buf(const POINTARRAY *pa, TWKB_GLOBALS *globals, TWKB
 	int64_t nextdelta[MAX_N_DIMS];
 	int npoints = 0;
 	size_t npoints_offset = 0;
+	uint32_t max_points_left = pa->npoints;
 
 	LWDEBUGF(2, "Entered %s", __func__);
 
@@ -132,7 +134,7 @@ static int ptarray_to_twkb_buf(const POINTARRAY *pa, TWKB_GLOBALS *globals, TWKB
 	{
 		/* Independent buffer to hold the coordinates, so we can put the npoints */
 		/* into the stream once we know how many points we actually have */
-		bytebuffer_init_with_size(&b, 3 * ndims * pa->npoints);
+		bytebuffer_init_with_size(&b, (size_t)3 * ndims * pa->npoints);
 		b_p = &b;
 	}
 	else
@@ -156,7 +158,7 @@ static int ptarray_to_twkb_buf(const POINTARRAY *pa, TWKB_GLOBALS *globals, TWKB
 	for ( i = 0; i < pa->npoints; i++ )
 	{
 		double *dbl_ptr = (double*)getPoint_internal(pa, i);
-		int diff = 0;
+		int64_t diff = 0;
 
 		/* Write this coordinate to the buffer as a varint */
 		for ( j = 0; j < ndims; j++ )
@@ -166,15 +168,18 @@ static int ptarray_to_twkb_buf(const POINTARRAY *pa, TWKB_GLOBALS *globals, TWKB
 			/* last accumulated point. This is important to not build up an */
 			/* accumulated error when rounding the coordinates */
 			nextdelta[j] = (int64_t) llround(globals->factor[j] * dbl_ptr[j]) - ts->accum_rels[j];
-			LWDEBUGF(4, "deltavalue: %d, ", nextdelta[j]);
+			LWDEBUGF(4, "deltavalue: %lld, ", nextdelta[j]);
 			diff += llabs(nextdelta[j]);
 		}
 
 		/* Skipping the first point is not allowed */
 		/* If the sum(abs()) of all the deltas was zero, */
 		/* then this was a duplicate point, so we can ignore it */
-		if ( i > minpoints && diff == 0 )
+		if ( i > 0 && diff == 0 &&  max_points_left > minpoints )
+		{
+			max_points_left--;
 			continue;
+		}
 
 		/* We really added a point, so... */
 		npoints++;
@@ -250,6 +255,17 @@ static int lwline_to_twkb_buf(const LWLINE *line, TWKB_GLOBALS *globals, TWKB_ST
 	return 0;
 }
 
+static int
+lwtriangle_to_twkb_buf(const LWTRIANGLE *tri, TWKB_GLOBALS *globals, TWKB_STATE *ts)
+{
+	LWDEBUGF(2, "Entered %s", __func__);
+	bytebuffer_append_uvarint(ts->geom_buf, (uint64_t)1);
+
+	/* Set the coordinates (do write npoints) */
+	ptarray_to_twkb_buf(tri->points, globals, ts, 1, 2);
+	return 0;
+}
+
 /******************************************************************
 * POLYGONS
 *******************************************************************/
@@ -375,6 +391,11 @@ static int lwgeom_to_twkb_buf(const LWGEOM *geom, TWKB_GLOBALS *globals, TWKB_ST
 			LWDEBUGF(4,"Type found is Linestring, %d", geom->type);
 			return lwline_to_twkb_buf((LWLINE*) geom, globals, ts);
 		}
+		case TRIANGLETYPE:
+		{
+			LWDEBUGF(4, "Type found is Triangle, %d", geom->type);
+			return lwtriangle_to_twkb_buf((LWTRIANGLE *)geom, globals, ts);
+		}
 		/* Polygon has 'nrings' and 'rings' elements */
 		case POLYGONTYPE:
 		{
@@ -391,13 +412,14 @@ static int lwgeom_to_twkb_buf(const LWGEOM *geom, TWKB_GLOBALS *globals, TWKB_ST
 			return lwmulti_to_twkb_buf((LWCOLLECTION*)geom, globals, ts);
 		}
 		case COLLECTIONTYPE:
+		case TINTYPE:
 		{
 			LWDEBUGF(4,"Type found is collection, %d", geom->type);
 			return lwcollection_to_twkb_buf((LWCOLLECTION*) geom, globals, ts);
 		}
 		/* Unknown type! */
 		default:
-			lwerror("Unsupported geometry type: %s [%d]", lwtype_name((geom)->type), (geom)->type);
+			lwerror("%s: Unsupported geometry type: %s", __func__, lwtype_name(geom->type));
 	}
 
 	return 0;
@@ -448,7 +470,7 @@ static int lwgeom_write_to_buffer(const LWGEOM *geom, TWKB_GLOBALS *globals, TWK
 		/* Reset bbox calculation */
 		child_state.bbox_max[i] = INT64_MIN;
 		child_state.bbox_min[i] = INT64_MAX;
-		/* Reset acumulated delta values to get absolute values on next point */
+		/* Reset accumulated delta values to get absolute values on next point */
 		child_state.accum_rels[i] = 0;
 	}
 
@@ -563,10 +585,13 @@ static int lwgeom_write_to_buffer(const LWGEOM *geom, TWKB_GLOBALS *globals, TWK
 * Convert LWGEOM to a char* in TWKB format. Caller is responsible for freeing
 * the returned array.
 */
-uint8_t*
-lwgeom_to_twkb_with_idlist(const LWGEOM *geom, int64_t *idlist, uint8_t variant,
-               int8_t precision_xy, int8_t precision_z, int8_t precision_m,
-               size_t *twkb_size)
+lwvarlena_t *
+lwgeom_to_twkb_with_idlist(const LWGEOM *geom,
+			   int64_t *idlist,
+			   uint8_t variant,
+			   int8_t precision_xy,
+			   int8_t precision_z,
+			   int8_t precision_m)
 {
 	LWDEBUGF(2, "Entered %s", __func__);
 	LWDEBUGF(2, "variant value %x", variant);
@@ -575,8 +600,6 @@ lwgeom_to_twkb_with_idlist(const LWGEOM *geom, int64_t *idlist, uint8_t variant,
 	TWKB_STATE ts;
 	bytebuffer_t geom_bytebuffer;
 
-	uint8_t *twkb;
-
 	memset(&ts, 0, sizeof(TWKB_STATE));
 	memset(&tg, 0, sizeof(TWKB_GLOBALS));
 
@@ -604,18 +627,15 @@ lwgeom_to_twkb_with_idlist(const LWGEOM *geom, int64_t *idlist, uint8_t variant,
 	bytebuffer_init_with_size(ts.geom_buf, 512);
 	lwgeom_write_to_buffer(geom, &tg, &ts);
 
-	twkb = bytebuffer_get_buffer_copy(ts.geom_buf, twkb_size);
+	lwvarlena_t *v = bytebuffer_get_buffer_varlena(ts.geom_buf);
 	bytebuffer_destroy_buffer(ts.geom_buf);
-	return twkb;
+	return v;
 }
 
-
-uint8_t*
-lwgeom_to_twkb(const LWGEOM *geom, uint8_t variant,
-               int8_t precision_xy, int8_t precision_z, int8_t precision_m,
-               size_t *twkb_size)
+lwvarlena_t *
+lwgeom_to_twkb(const LWGEOM *geom, uint8_t variant, int8_t precision_xy, int8_t precision_z, int8_t precision_m)
 {
-	return lwgeom_to_twkb_with_idlist(geom, NULL, variant, precision_xy, precision_z, precision_m, twkb_size);
+	return lwgeom_to_twkb_with_idlist(geom, NULL, variant, precision_xy, precision_z, precision_m);
 }
 
 
